@@ -5,7 +5,7 @@
  * This file defines Turba's external API interface. Other applications can
  * interact with Turba through this API.
  *
- * Copyright 2009-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you did
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -190,7 +190,7 @@ class Turba_Api extends Horde_Registry_Api
     public function browse($path = '',
                           $properties = array('name', 'icon', 'browseable'))
     {
-        global $injector, $session;
+        global $injector, $registry, $session;
 
         // Strip off the application name if present
         if (substr($path, 0, 5) == 'turba') {
@@ -269,17 +269,14 @@ class Turba_Api extends Horde_Registry_Api
                     // No backends are configured to provide shares
                     return array();
                 }
-                $addressbooks = $injector->getInstance('Turba_Shares')->listShares($parts[0], array(
-                    'attributes' => $parts[0],
-                    'perm' => Horde_Perms::READ
-                ));
-
-                /* The last check returns all addressbooks for the requested
-                 * user, but that does not mean the requesting user has access
-                 * to them.
-                 * Filter out those address books for which the requesting
-                 * user has no access. */
-                $addressbooks = Turba::permissionsFilter($addressbooks);
+                $addressbooks = $injector->getInstance('Turba_Shares')
+                    ->listShares(
+                        $registry->getAuth(),
+                        array(
+                            'attributes' => $parts[0],
+                            'perm' => Horde_Perms::READ
+                        )
+                    );
             }
 
             $curpath = 'turba/' . $parts[0] . '/';
@@ -356,7 +353,7 @@ class Turba_Api extends Horde_Registry_Api
                 }
                 if (in_array('contentlength', $properties)) {
                     try {
-                        $data = $this->export($contact->getValue('__uid'), 'text/x-vcard', $contact->getSource());
+                        $data = $this->export($contact->getValue('__uid'), 'text/x-vcard', $contact->getSource(), null, array('skip_empty' => true));
                     } catch (Turba_Exception $e) {
                         $data = '';
                     }
@@ -387,7 +384,7 @@ class Turba_Api extends Horde_Registry_Api
             $contact = $driver->getObject($parts[2]);
 
             $result = array(
-                'data' => $this->export($contact->getValue('__uid'), 'text/x-vcard', $contact->getSource()),
+                'data' => $this->export($contact->getValue('__uid'), 'text/x-vcard', $contact->getSource(), null, array('skip_empty' => true)),
                 'mimetype' => 'text/x-vcard'
             );
             $modified = $this->_modified($contact->getValue('__uid'), $parts[1]);
@@ -546,19 +543,20 @@ class Turba_Api extends Horde_Registry_Api
      * Essentially a wrapper around listBy(), but returns an array containing
      * all adds, edits, and deletions.
      *
-     * @param integer $start     The starting timestamp
-     * @param integer $end       The ending timestamp.
-     * @param boolean $isModSeq  If true, $start and $end are modification
-     *                           sequences and not timestamps. @since 4.1.1
+     * @param integer $start         The starting timestamp
+     * @param integer $end           The ending timestamp.
+     * @param boolean $isModSeq      If true, $start and $end are modification
+     *                               sequences and not timestamps. @since 4.1.1
+     * @param string|array $sources  The sources to check. @since 4.2.0
      *
      * @return array  A hash with 'add', 'modify' and 'delete' arrays.
      */
-    public function getChanges($start, $end, $isModSeq = false)
+    public function getChanges($start, $end, $isModSeq = false, $sources = null)
     {
         return array(
-            'add' => $this->listBy('add', $start, null, $end, $isModSeq),
-            'modify' => $this->listBy('modify', $start, null, $end, $isModSeq),
-            'delete' => $this->listBy('delete', $start, null, $end, $isModSeq)
+            'add' => $this->listBy('add', $start, $sources, $end, $isModSeq),
+            'modify' => $this->listBy('modify', $start, $sources, $end, $isModSeq),
+            'delete' => $this->listBy('delete', $start, $sources, $end, $isModSeq)
         );
     }
 
@@ -566,15 +564,16 @@ class Turba_Api extends Horde_Registry_Api
      * Return all changes occuring between the specified modification
      * sequences.
      *
-     * @param integer $start  The starting modseq.
-     * @param integer $end    The ending modseq.
+     * @param integer $start         The starting modseq.
+     * @param integer $end           The ending modseq.
+     * @param string|array $sources  The sources to check. @since 4.2.0
      *
      * @return array  The changes @see getChanges()
      * @since 4.1.1
      */
-    public function getChangesByModSeq($start, $end)
+    public function getChangesByModSeq($start, $end, $sources = null)
     {
-        return $this->getChanges($start, $end, true);
+        return $this->getChanges($start, $end, true, $sources);
     }
 
     /**
@@ -619,12 +618,20 @@ class Turba_Api extends Horde_Registry_Api
     /**
      * Return the largest modification sequence from the history backend.
      *
+     * @param string $id  Addressbook id to return highest MODSEQ for. If
+     *                    null, the highest MODSEQ across all addressbooks is
+     *                    returned. @since 4.2.0
+     *
      * @return integer  The modseq.
      * @since 4.1.1
      */
-    public function getHighestModSeq()
+    public function getHighestModSeq($id = null)
     {
-        return $GLOBALS['injector']->getInstance('Horde_History')->getHighestModSeq('turba');
+        $parent = 'turba';
+        if (!empty($id)) {
+            $parent .= ':' . $id;
+        }
+        return $GLOBALS['injector']->getInstance('Horde_History')->getHighestModSeq($parent);
     }
 
     /**
@@ -676,10 +683,6 @@ class Turba_Api extends Horde_Registry_Api
             throw new Turba_Exception(_("Permission denied"));
         }
 
-        // Create a category manager.
-        $cManager = new Horde_Prefs_CategoryManager();
-        $categories = $cManager->get();
-
         if (!($content instanceof Horde_Icalendar_Vcard)) {
             switch ($contentType) {
             case 'activesync':
@@ -719,13 +722,7 @@ class Turba_Api extends Horde_Registry_Api
                                 continue;
                             }
 
-                            $result = $driver->add($content);
-                            if (!empty($content['category']) &&
-                                !in_array($content['category'], $categories)) {
-                                $cManager->add($content['category']);
-                                $categories[] = $content['category'];
-                            }
-                            $ids[] = $result;
+                            $ids[] = $driver->add($content);
                         }
                     }
 
@@ -750,20 +747,20 @@ class Turba_Api extends Horde_Registry_Api
 
         // We can't use $object->setValue() here since that cannot be used
         // with composite fields.
-        if (Horde::hookExists('encode_attribute', 'turba')) {
+        $hooks = $injector->getInstance('Horde_Core_Hooks');
+        if ($hooks->hookExists('encode_attribute', 'turba')) {
             foreach ($content as $attribute => &$value) {
                 try {
-                    $value = Horde::callHook('encode_attribute', array($attribute, $value, null, null), 'turba');
+                    $value = $hooks->callHook(
+                        'encode_attribute',
+                        'turba',
+                        array($attribute, $value, null, null)
+                    );
                 } catch (Turba_Exception $e) {}
             }
         }
         $result = $driver->add($content);
 
-        if (!empty($content['category']) &&
-            !in_array($content['category'], $categories)) {
-            $cManager->add($content['category']);
-        }
-
         return $driver->getObject($result)->getValue('__uid');
     }
 
@@ -771,20 +768,30 @@ class Turba_Api extends Horde_Registry_Api
      * Export a contact, identified by UID, in the requested contentType.
      *
      * @param string $uid            Identify the contact to export.
-     * @param mixed $contentType     What format should the data be in?  Either
-     *                               a string with one of: - text/directory -
-     *                               text/vcard - text/x-vcard The first two
-     *                               produce a vcard3.0 (rfc2426), the second
-     *                               produces a vcard in old 2.1 format
-     *                               defined by imc.org Also supports a raw
-     *                               array
+     * @param mixed $contentType     What format should the data be in?
+     *   - text/directory: Returns RFC2426 vcard3.0
+     *   - text/vcard:     Returns RFC2426 vcard3.0
+     *   - text/x-vcard:   Returns imc.org vcard 2.1 format.
+     *   - array:          Returns a raw array
+     *   - activesync:     Returns a Horde_ActiveSync_Message_Contact:: object
      * @param string|array $sources  The source(s) from which the contact will
      *                               be exported.
      * @param array $fields          Hash of field names and
      *                               Horde_SyncMl_Property properties with the
      *                               requested fields.
      * @param array $options         Any additional options to be passed to the
-     *                               exporter.
+     *                               exporter. Currently supported:
+     *   - skip_empty:       (boolean) {text/vcard or text/x-vcard} Set to
+     *                       true to not output empty properties.
+     *                       DEFAULT: false.
+     *   - protocolversion:  (float) {activesync} The EAS version to support
+     *                       DEFAULT: 2.5
+     *   - bodyprefs:        (array) {activesync} A BODYPREFERENCE array.
+     *                       DEFAULT: none (No body prefs enforced).
+     *   - truncation:       (integer) {activesync} Truncate event body to this
+     *                       length.
+     *                       DEFAULT: none (No truncation).
+     *
      *
      * @return mixed  The requested data.
      * @throws Turba_Exception
@@ -823,7 +830,7 @@ class Turba_Api extends Horde_Registry_Api
             case 'text/directory':
                 $export = '';
                 foreach ($result->objects as $obj) {
-                    $vcard = $sdriver->tovCard($obj, $version, $fields);
+                    $vcard = $sdriver->tovCard($obj, $version, $fields, !empty($options['skip_empty']));
                     /* vCards are not enclosed in
                      * BEGIN:VCALENDAR..END:VCALENDAR.  Export the individual
                      * cards instead. */
@@ -1169,15 +1176,20 @@ class Turba_Api extends Horde_Registry_Api
                     }
                 }
 
-                $search = $sdriver->search(
-                    $criteria,
-                    Turba::getPreferredSortOrder(),
-                    'OR',
-                    $opts['returnFields'],
-                    $opts['customStrict'],
-                    $opts['matchBegin'],
-                    $opts['count_only']
-                );
+                try {
+                    $search = $sdriver->search(
+                        $criteria,
+                        Turba::getPreferredSortOrder(),
+                        'OR',
+                        $opts['returnFields'],
+                        $opts['customStrict'],
+                        $opts['matchBegin'],
+                        $opts['count_only']
+                    );
+                } catch (Exception $e) {
+                    continue;
+                }
+
                 if ($opts['count_only']) {
                     $results += $search;
 
@@ -1242,21 +1254,31 @@ class Turba_Api extends Horde_Registry_Api
 
                         $email = new Horde_Mail_Rfc822_List();
 
+                        $display_name = ($ob->hasValue('name') || !isset($ob->driver->alternativeName))
+                            ? Turba::formatName($ob)
+                            : $ob->getValue($ob->driver->alternativeName);
+
                         foreach (array_keys($att) as $key) {
                             if ($ob->getValue($key) &&
                                 isset($attributes[$key]) &&
                                 ($attributes[$key]['type'] == 'email')) {
-                                // Multiple addresses support
-                                $email->add($rfc822->parseAddressList($ob->getValue($key), array(
-                                    'limit' => (isset($attributes[$key]['params']) && is_array($attributes[$key]['params']) && !empty($attributes[$key]['params']['allow_multi'])) ? 0 : 1
-                                )));
+                                $e_val = $ob->getValue($key);
+
+                                /* Bug #12480: Don't return email if it
+                                 * doesn't contain the search string, since
+                                 * an entry can contain multiple e-mail
+                                 * fields. Return all e-mails if it
+                                 * occurrs in the name. */
+                                if ((stripos($e_val, $name) !== false) ||
+                                    (stripos($display_name, $name) !== false)) {
+                                    // Multiple addresses support
+                                    $email->add($rfc822->parseAddressList($e_val, array(
+                                        'limit' => (isset($attributes[$key]['params']) && is_array($attributes[$key]['params']) && !empty($attributes[$key]['params']['allow_multi'])) ? 0 : 1
+                                    )));
+                                }
                             }
                         }
 
-                        $display_name = ($ob->hasValue('name') || !isset($ob->driver->alternativeName))
-                            ? Turba::formatName($ob)
-                            : $ob->getValue($ob->driver->alternativeName);
-
                         if (count($email)) {
                             foreach ($email as $val) {
                                 $seen_key = trim(Horde_String::lower($display_name)) . '/' . Horde_String::lower($val->bare_address);
@@ -1564,7 +1586,7 @@ class Turba_Api extends Horde_Registry_Api
                                   array $fields = array(),
                                   $matchBegin = false)
     {
-        $abook = $GLOBALS['conf']['client']['addressbook'];
+        $abook = $this->getClientSource();
         return $this->search(
             $names,
             array('sources' => array($abook),
@@ -1732,7 +1754,7 @@ class Turba_Api extends Horde_Registry_Api
             try {
                 $list = $sdriver->search($criterium, null, 'AND', array(), $strict ? array('email') : array());
             } catch (Turba_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
 
             if ($list instanceof Turba_List) {
@@ -1821,12 +1843,12 @@ class Turba_Api extends Horde_Registry_Api
     public function getSourcesConfig($filter = array())
     {
         $results = array();
-        $turba_sources = Horde::loadConfiguration('backends.php', 'cfgSources', 'turba');
 
-        foreach ($turba_sources as $key => $source) {
-            if (!empty($filter)) {
-                if (!empty($source[current(array_keys($filter))]) &&
-                    $source[current(array_keys($filter))] == current($filter)) {
+        if (!empty($filter)) {
+            foreach (Turba::availableSources() as $key => $source) {
+                $curr = current(array_keys($filter));
+                if (!empty($source[$curr]) &&
+                    ($source[$curr] == current($filter))) {
                     $results[$key] = $source;
                 }
             }
@@ -1898,9 +1920,9 @@ class Turba_Api extends Horde_Registry_Api
             try {
                 $results = $db[$key]->selectAssoc($sql);
             } catch (Horde_Db_Exception $e) {
-                Horde::logMessage($e);
-                throw new Horde_Exception_Wrapped($e);
+                throw new Horde_Exception(_("Server error when performing search."));
             }
+
             foreach ($results as $id => $name) {
                 $groups[$key . ':' . $id] = $name;
             }
@@ -1929,8 +1951,7 @@ class Turba_Api extends Horde_Registry_Api
             try {
                 $results = $db->selectAll($sql);
             } catch (Horde_Db_Exception $e) {
-                Horde::logMessage($e);
-                throw new Turba_Exception($e);
+                throw new Horde_Exception(_("Server error when performing search."));
             }
 
             foreach ($results as $row) {
@@ -1996,8 +2017,7 @@ class Turba_Api extends Horde_Registry_Api
             unset($ret['lname']);
             return $ret;
         } catch (Horde_Db_Exception $e) {
-            Horde::logMessage($e);
-            throw new Turba_Exception($e);
+            throw new Horde_Exception(_("Server error when performing search."));
         }
     }
 
@@ -2078,8 +2098,7 @@ class Turba_Api extends Horde_Registry_Api
             try {
                 $results = $db[$newSource]->selectOne($sql);
             } catch (Horde_Db_Exception $e) {
-                Horde::logMessage($e);
-                throw new Horde_Exception_Wrapped($e);
+                throw new Horde_Exception(_("Server error when performing search."));
             }
 
             // Sub-Lists are treated as sub groups the best that we can...
@@ -2096,6 +2115,73 @@ class Turba_Api extends Horde_Registry_Api
         return array_keys($users);
     }
 
+    /**
+     * Create a new addressbook
+     *
+     * @param string $name   The display name for the addressbook.
+     * @param array  $params Any addtional parameters needed.
+     *     - synchronize:   (boolean) If true, add address book to the list of
+     *                                address books to syncronize.
+     *                      DEFAULT: false (do not add to the list).
+     *                      @since 4.2.1
+     *
+     * @return string  The new addressbook's id (share name).
+     * @since 4.2.0
+     */
+    public function addAddressbook($name, array $params = array())
+    {
+        $share_name = strval(new Horde_Support_Randomid());
+        $share = Turba::createShare($share_name, array('name' => $name));
+        $name = $share->getName();
+        if (!empty($params['synchronize'])) {
+            $sync = @unserialize($prefs->getValue('sync_books'));
+            $sync[] = $name;
+            $prefs->setValue('sync_books', serialize($sync));
+        }
+
+        return $name;
+    }
+
+    /**
+     * Delete the specified addressbook.
+     *
+     * @param string $id  The addressbook id.
+     * @since 4.2.0
+     */
+    public function deleteAddressbook($id)
+    {
+        $share = $GLOBALS['injector']
+            ->getInstance('Turba_Factory_Driver')
+            ->create($id);
+
+        $GLOBALS['injector']
+            ->getInstance('Turba_Shares')
+            ->removeShare($share);
+    }
+
+    /**
+     * Update an existing addressbook's name or description.
+     *
+     * @param string $id    The addressbook id.
+     * @param array  $info  The info to change:
+     *   - name: The addressbook's display name.
+     *   - desc: The addressbook's description.
+     *
+     * @since 4.2.0
+     */
+    public function updateAddressbook($id, array $info)
+    {
+        $share = $GLOBALS['injector']->getInstance('Turba_Factory_Driver')->create($id);
+        if (!empty($info['name'])) {
+            $share->set('name', $info['name']);
+        }
+        if (!empty($info['desc'])) {
+            $share->set('desc', $info['desc']);
+        }
+
+        $share->save();
+    }
+
     /* Helper methods. */
 
     /**
@@ -2137,4 +2223,108 @@ class Turba_Api extends Horde_Registry_Api
         return $sources;
     }
 
+    /**
+     * Retrieve the list of used tag_names, tag_ids and the total number
+     * of resources that are linked to that tag.
+     *
+     * @param array $tags  An optional array of tag_ids. If omitted, all tags
+     *                     will be included.
+     *
+     * @return array  An array containing tag_name, and total
+     */
+    public function listTagInfo($tags = null, $user = null)
+    {
+        return $GLOBALS['injector']->getInstance('Turba_Tagger')
+            ->getTagInfo($tags, 500, null, $user);
+    }
+
+    /**
+     * SearchTags API:
+     * Returns an application-agnostic array (useful for when doing a tag search
+     * across multiple applications)
+     *
+     * The 'raw' results array can be returned instead by setting $raw = true.
+     *
+     * @param array $names           An array of tag_names to search for.
+     * @param integer $max           The maximum number of resources to return.
+     * @param integer $from          The number of the resource to start with.
+     * @param string $resource_type  The resource type [bookmark, '']
+     * @param string $user           Restrict results to resources owned by $user.
+     * @param boolean $raw           Return the raw data?
+     *
+     * @return array An array of results:
+     * <pre>
+     *  'title'    - The title for this resource.
+     *  'desc'     - A terse description of this resource.
+     *  'view_url' - The URL to view this resource.
+     *  'app'      - The Horde application this resource belongs to.
+     *  'icon'     - URL to an image.
+     * </pre>
+     */
+    public function searchTags($names, $max = 10, $from = 0,
+                               $resource_type = '', $user = null, $raw = false)
+    {
+        global $injector, $registry;
+
+        $results = $injector
+            ->getInstance('Turba_Tagger')
+            ->search(
+                $names,
+                array('user' => $user));
+
+        // Check for error or if we requested the raw data array.
+        if ($raw) {
+            return $results;
+        }
+        $return = array();
+        foreach ($results as $contact_uid) {
+            try {
+                $driver = $injector->getInstance('Turba_Factory_Driver');
+                foreach ($this->_getSources($sources) as $source) {
+                    $sdriver = $driver->create($source);
+                    if (!$sdriver->hasPermission(Horde_Perms::READ)) {
+                        continue;
+                    }
+                    $result = $sdriver->search(array('__uid' => $contact_uid));
+                    if (count($result) == 0) {
+                        continue;
+                    } elseif (count($result) > 1) {
+                        throw new Turba_Exception(sprintf("Internal Horde Error: multiple Turba objects with same objectId %s.", $uid));
+                    }
+                    foreach ($result->objects as $obj) {
+                        $return[] = array(
+                            'title' => $obj->getValue('name'),
+                            'desc' => $obj->getValue('name'),
+                            'view_url' => $obj->url,
+                            'app' => 'turba',
+                            'icon' => $this->_getContactImageUrl($obj)
+                        );
+                    }
+                }
+            } catch (Exception $e) {
+            }
+        }
+
+        return $return;
+    }
+
+
+    protected function _getContactImageUrl($obj)
+    {
+        if ($photo = $obj->getValue('photo')) {
+            try {
+                $img = $GLOBALS['injector']->getInstance('Horde_Core_Factory_Image')->create();
+                $img->loadString($photo['load']['data']);
+                $img->resize(50, 50, true);
+                $data = $img->raw(true);
+                $type = $img->getContentType();
+            } catch (Horde_Image_Exception $e) {
+                $data = $photo['load']['data'];
+                $type = $obj->getValue('phototype');
+            }
+
+            return Horde_Url_Data::create($type, $data);
+        }
+    }
+
 }
