@@ -1,23 +1,32 @@
 <?php
+/**
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
+ *
+ * See the enclosed file LICENSE for license information (ASL).  If you did
+ * did not receive this file, see http://www.horde.org/licenses/apache.
+ *
+ * @category  Horde
+ * @copyright 2010-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/apache ASL
+ * @package   Turba
+ */
+
 /**
  * Turba application API.
  *
  * This file defines Horde's core API interface. Other core Horde libraries
  * can interact with Turba through this API.
  *
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
- *
- * See the enclosed file COPYING for license information (APL). If you
- * did not receive this file, see http://www.horde.org/licenses/apl.html.
- *
- * @category Horde
- * @license  http://www.horde.org/licenses/apl.html APL
- * @package  Turba
+ * @author    Michael Slusarz <slusarz@horde.org?
+ * @category  Horde
+ * @copyright 2010-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/apache ASL
+ * @package   Turba
  */
 
 /* Determine the base directories. */
 if (!defined('TURBA_BASE')) {
-    define('TURBA_BASE', __DIR__ . '/..');
+    define('TURBA_BASE', realpath(__DIR__ . '/..'));
 }
 
 if (!defined('HORDE_BASE')) {
@@ -26,7 +35,7 @@ if (!defined('HORDE_BASE')) {
     if (file_exists(TURBA_BASE . '/config/horde.local.php')) {
         include TURBA_BASE . '/config/horde.local.php';
     } else {
-        define('HORDE_BASE', TURBA_BASE . '/..');
+        define('HORDE_BASE', realpath(TURBA_BASE . '/..'));
     }
 }
 
@@ -34,6 +43,9 @@ if (!defined('HORDE_BASE')) {
  * Horde_Registry_Application::). */
 require_once HORDE_BASE . '/lib/core.php';
 
+use Sabre\CalDAV;
+use Sabre\CardDAV;
+
 class Turba_Application extends Horde_Registry_Application
 {
     /**
@@ -45,7 +57,7 @@ class Turba_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (4.1.3)';
+    public $version = 'H5 (4.2.1)';
 
     /**
      */
@@ -53,7 +65,8 @@ class Turba_Application extends Horde_Registry_Application
     {
         /* Add Turba-specific factories. */
         $factories = array(
-            'Turba_Shares' => 'Turba_Factory_Shares'
+            'Turba_Shares' => 'Turba_Factory_Shares',
+            'Turba_Tagger' => 'Turba_Factory_Tagger'
         );
 
         foreach ($factories as $key => $val) {
@@ -72,8 +85,21 @@ class Turba_Application extends Horde_Registry_Application
      */
     protected function _init()
     {
+        global $injector, $registry, $session;
+
+        /* For now, autoloading the Content_* classes depend on there being a
+         * registry entry for the 'content' application that contains at least
+         * the fileroot entry. */
+        $injector->getInstance('Horde_Autoloader')
+            ->addClassPathMapper(
+                new Horde_Autoloader_ClassPathMapper_Prefix('/^Content_/', $registry->get('fileroot', 'content') . '/lib/'));
+
+        if (!class_exists('Content_Tagger')) {
+            throw new Horde_Exception(_("The Content_Tagger class could not be found. Make sure the Content application is installed."));
+        }
+
         // Turba source and attribute configuration.
-        $attributes = Horde::loadConfiguration('attributes.php', 'attributes', 'turba');
+        $attributes = $registry->loadConfigFile('attributes.php', 'attributes', 'turba')->config['attributes'];
         $cfgSources = Turba::availableSources();
 
         /* UGLY UGLY UGLY - we should NOT be using this as a global
@@ -84,7 +110,7 @@ class Turba_Application extends Horde_Registry_Application
         foreach ($cfgSources as $key => $cfg) {
             if (!empty($cfg['use_shares'])) {
                 // Create a share instance.
-                $GLOBALS['session']->set('turba', 'has_share', true);
+                $session->set('turba', 'has_share', true);
                 $cfgSources = Turba::getConfigFromShares($cfgSources);
                 break;
             }
@@ -95,7 +121,7 @@ class Turba_Application extends Horde_Registry_Application
 
         // Build the directory sources select widget.
         if (empty(Turba::$source)) {
-            if (!(Turba::$source = $GLOBALS['session']->get('turba', 'source'))) {
+            if (!(Turba::$source = $session->get('turba', 'source'))) {
                 Turba::$source = Turba::getDefaultAddressbook();
             }
             Turba::$source = Horde_Util::getFormData('source', Turba::$source);
@@ -115,7 +141,7 @@ class Turba_Application extends Horde_Registry_Application
         if (empty($cfgSources[Turba::$source]['browse'])) {
             Turba::$source = Turba::getDefaultAddressbook();
         }
-        $GLOBALS['session']->set('turba', 'source', Turba::$source);
+        $session->set('turba', 'source', Turba::$source);
 
         $GLOBALS['addSources'] = Turba::getAddressBooks(Horde_Perms::EDIT, array('require_add' => true));
         $GLOBALS['copymoveSources'] = $GLOBALS['addSources'];
@@ -148,6 +174,22 @@ class Turba_Application extends Horde_Registry_Application
         return $perms;
     }
 
+    /**
+     */
+    public function getInitialPage()
+    {
+        global $registry;
+
+        switch ($registry->getView()) {
+        case $registry::VIEW_SMARTMOBILE:
+            return strval(Horde::url('smartmobile.php')->setRaw(true));
+            break;
+
+        default:
+            return null;
+        }
+    }
+
     /**
      */
     public function menu($menu)
@@ -301,7 +343,7 @@ class Turba_Application extends Horde_Registry_Application
                         ->getInstance('Turba_Factory_Driver')
                         ->create($source);
                 } catch (Turba_Exception $e) {
-                    Horde::logMessage($e, 'ERR');
+                    Horde::log($e, 'ERR');
                 }
 
                 try {
@@ -309,7 +351,7 @@ class Turba_Application extends Horde_Registry_Application
                 } catch (Turba_Exception_NotSupported $e) {
                     continue;
                 } catch (Turba_Exception $e) {
-                    Horde::logMessage($e, 'ERR');
+                    Horde::log($e, 'ERR');
                     throw new Turba_Exception(sprintf(_("There was an error removing an address book for %s"), $user));
                 }
             }
@@ -341,7 +383,7 @@ class Turba_Application extends Horde_Registry_Application
             } catch (Turba_Exception_NotSupported $e) {
                 continue;
             } catch (Turba_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
                 throw new Turba_Exception(sprintf(_("There was an error removing an address book for %s"), $user));
             }
         }
@@ -354,7 +396,7 @@ class Turba_Application extends Horde_Registry_Application
                 $share->removeUser($user);
             }
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
             throw new Turba_Exception(sprintf(_("There was an error removing an address book for %s"), $user));
         }
     }
@@ -447,19 +489,13 @@ class Turba_Application extends Horde_Registry_Application
                 throw new Turba_Exception(_("You do not have permission to view this contact."));
             }
 
-            try {
-                $vfs = $object->vfsInit();
-            } catch (Horde_Vfs_Exception $e) {
-                throw new Turba_Exception(_("Data cannot be downloaded as no VFS driver is configured."));
-            }
-
             try {
                 return array(
-                    'data' => $vfs->read(Turba::VFS_PATH . '/' . $object->getValue('__uid'), $vars->file),
+                    'data' => $object->vfsInit()->read(Turba::VFS_PATH . '/' . $object->getValue('__uid'), $vars->file),
                     'name' => $vars->file
                 );
             } catch (Horde_Vfs_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
                 throw new Turba_Exception(sprintf(_("Access denied to %s"), $vars->file));
             }
 
@@ -491,6 +527,7 @@ class Turba_Application extends Horde_Registry_Application
             foreach ($sources as $source => $objectkeys) {
                 /* Create a Turba storage instance. */
                 $driver = $tfd->create($source);
+                $blobs = $driver->getBlobs();
 
                 /* Get the full, sorted contact list. */
                 try {
@@ -511,17 +548,19 @@ class Turba_Application extends Horde_Registry_Application
                     } else {
                         $row = array();
                         foreach ($fields as $field) {
-                            if (substr($field, 0, 2) != '__') {
-                                $attribute = $ob->getValue($field);
-                                if ($attributes[$field]['type'] == 'date') {
-                                    $row[$field] = strftime('%Y-%m-%d', $attribute);
-                                } elseif ($attributes[$field]['type'] == 'time') {
-                                    $row[$field] = strftime('%R', $attribute);
-                                } elseif ($attributes[$field]['type'] == 'datetime') {
-                                    $row[$field] = strftime('%Y-%m-%d %R', $attribute);
-                                } else {
+                            if (substr($field, 0, 2) == '__' ||
+                                isset($blobs[$field])) {
+                                continue;
+                            }
+                            $attribute = $ob->getValue($field);
+                            if ($attributes[$field]['type'] == 'date') {
+                                $row[$field] = strftime('%Y-%m-%d', $attribute);
+                            } elseif ($attributes[$field]['type'] == 'time') {
+                                $row[$field] = strftime('%R', $attribute);
+                            } elseif ($attributes[$field]['type'] == 'datetime') {
+                                $row[$field] = strftime('%Y-%m-%d %R', $attribute);
+                            } else {
                                 $row[$field] = Horde_String::convertCharset($attribute, 'UTF-8', $params['charset']);
-                                }
                             }
                         }
                         $data[] = $row;
@@ -614,7 +653,7 @@ class Turba_Application extends Horde_Registry_Application
             'Assistant\'s Name' => 'assistant',
             'Birthday' => 'birthday',
             'Business Address PO Box' => 'workPOBox',
-            'Categories' => 'category',
+            'Categories' => '__tags',
             'Children' => 'children',
             'E-mail Address' => 'email',
             'Home Address PO Box' => 'homePOBox',
@@ -635,10 +674,30 @@ class Turba_Application extends Horde_Registry_Application
      */
     public function davGetCollections($user)
     {
-        $dav = $GLOBALS['injector']
-            ->getInstance('Horde_Dav_Storage');
+        global $injector, $registry;
+
+        $dav = $injector->getInstance('Horde_Dav_Storage');
+        $factory = $injector->getInstance('Turba_Shares');
         $books = array();
         foreach (Turba::getAddressBooks(Horde_Perms::SHOW) as $id => $book) {
+            // Ugly hack! There is currently no clean way to retrieve address
+            // books that the user "owns", or to find out if a SQL/LDAP/Kolab
+            // address book contains per-user or global contacts.
+            if ($book['type'] == 'share') {
+                $share = $factory->getShare($id);
+                if (($user == '-system-' && strlen($share->get('owner'))) ||
+                    ($user != '-system-' &&
+                     $user != $share->get('owner') &&
+                     $user != $registry->getAuth())) {
+                    continue;
+                }
+            }
+            if ($user == '-system-' &&
+                ($book['type'] == 'facebook' ||
+                 $book['type'] == 'favourites' ||
+                 $book['type'] == 'vbook')) {
+                continue;
+            }
             try {
                 $id = $dav->getExternalCollectionId($id, 'contacts') ?: $id;
             } catch (Horde_Dav_Exception $e) {
@@ -648,8 +707,8 @@ class Turba_Application extends Horde_Registry_Application
                 'uri' => $id,
                 'principaluri' => 'principals/' . $user,
                 '{DAV:}displayname' => $book['title'],
-                '{urn:ietf:params:xml:ns:carddav}supported-address-data'
-                    => new Sabre\CardDAV\Property\SupportedAddressData(
+                '{' . CardDAV\Plugin::NS_CARDDAV . '}supported-address-data'
+                    => new CardDAV\Property\SupportedAddressData(
                         array(
                             array(
                                 'contentType' => 'text/directory',
@@ -733,7 +792,8 @@ class Turba_Application extends Horde_Registry_Application
         } catch (Horde_Dav_Exception $e) {
         }
 
-        $data = $driver->tovCard($contact, '3.0')->exportvCalendar();
+        $data = $driver->tovCard($contact, '3.0', null, true)
+            ->exportvCalendar();
 
         return array(
             'id' => $id,
