@@ -2,7 +2,7 @@
 /**
  * Turba search.php.
  *
- * Copyright 2000-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2000-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -72,7 +72,7 @@ try {
 if ($driver) {
     $map = $driver->getCriteria();
     $do_search = false;
-    $submitted = Horde_Util::getPost('search');
+    $submitted = Horde_Util::getFormData('search');
     $browsable = !empty($cfgSources[$source]['browse']) && !empty($submitted);
     switch ($search_mode) {
     case 'advanced':
@@ -123,47 +123,44 @@ if ($driver) {
     if ($do_search) {
         if ($vars->save_vbook) {
             try {
-                $injector->getInstance('Horde_Token')->validate(
-                    $vars->token,
-                    'turba.search'
-                );
-
-                /* We create the vbook and redirect before we try to search
-                 * since we are not displaying the search results on this page
-                 * anyway. */
-                $vname = $vars->vbook_name;
-                if (empty($vname)) {
-                    $notification->push(_("You must provide a name for virtual address books."), 'horde.error');
-                    Horde::url('search.php', true)->redirect();
-                }
-
-                /* Create the vbook. */
-                $params = array(
-                    'name' => $vname,
-                    'params' => serialize(array(
-                        'type' => 'vbook',
-                        'source' => $source,
-                        'criteria' => $criteria
-                    ))
-                );
-
-                try {
-                    $share = Turba::createShare(strval(new Horde_Support_Randomid()), $params);
-                    $vid = $share->getName();
-                } catch (Horde_Share_Exception $e) {
-                    $notification->push(sprintf(_("There was a problem creating the virtual address book: %s"), $e->getMessage()), 'horde.error');
-                    Horde::url('search.php', true)->redirect();
-                }
-
-                $notification->push(sprintf(_("Successfully created virtual address book \"%s\""), $vname), 'horde.success');
-
-                Horde::url('browse.php', true)
-                    ->add('source', $vid)
-                    ->redirect();
-            } catch (Horde_Token_Exception $e) {
+                $session->checkToken($vars->token);
+            } catch (Horde_Exception $e) {
                 $notification->push($e);
                 Horde::url('search.php', true)->redirect();
             }
+
+            /* We create the vbook and redirect before we try to search
+             * since we are not displaying the search results on this page
+             * anyway. */
+            $vname = $vars->vbook_name;
+            if (empty($vname)) {
+                $notification->push(_("You must provide a name for virtual address books."), 'horde.error');
+                Horde::url('search.php', true)->redirect();
+            }
+
+            /* Create the vbook. */
+            $params = array(
+                'name' => $vname,
+                'params' => serialize(array(
+                    'type' => 'vbook',
+                    'source' => $source,
+                    'criteria' => $criteria
+                ))
+            );
+
+            try {
+                $share = Turba::createShare(strval(new Horde_Support_Randomid()), $params);
+                $vid = $share->getName();
+            } catch (Horde_Share_Exception $e) {
+                $notification->push(sprintf(_("There was a problem creating the virtual address book: %s"), $e->getMessage()), 'horde.error');
+                Horde::url('search.php', true)->redirect();
+            }
+
+            $notification->push(sprintf(_("Successfully created virtual address book \"%s\""), $vname), 'horde.success');
+
+            Horde::url('browse.php', true)
+                ->add('source', $vid)
+                ->redirect();
         }
 
         /* Perform a search. */
@@ -232,7 +229,7 @@ if ($search_mode != 'duplicate' && $session->get('turba', 'has_share')) {
     $vbookView->hasShare = true;
     $vbookView->shareSources = $shareSources;
     $vbookView->source = $source;
-    $vbookView->token = $injector->getInstance('Horde_Token')->get('turba.search');
+    $vbookView->token = $session->getToken();
     $page_output->addInlineScript('$(\'vbook_name\').observe(\'keyup\', function() { $(\'save-vbook\').checked = !!$F(\'vbook_name\'); });');
 }
 
